;================================================================================
;
;chanbctl: TIA-232 CHANNEL B RECEIVER CONTROL
;
;	
;	This function enables or disables the UART's B-channel receiver.  Unless
;	enabled, the channel will not accept data from a foreign source & will
;	keep CTS deasserted.
;
;	Preparatory Ops : see programming example
;
;	Register Returns: .A: used
;	                  .B: entry value
;	                  .X: entry value
;	                  .Y: entry value
;	                  DB: entry value
;	                  DP: entry value
;	                  PB: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> entry value
;	           ||||||+> 0
;	           |||+++> entry values
;	           ||+> 1
;	           |+> entry value
;	           +> undefined
;
;	Examples: clc              ;disable
;	          jsr chanbctl
;
;	          sec              ;enable
;	          jsr chanbctl
;	
;
chanbctl shorta
         bcc disabchb          ;disable
;
;================================================================================
;
;enabchb: internal function

;
enabchb  lda #nxpcrrsa|nxpcrrxe ;assert RTS & enable RxD
         .byte bitabs
;
;================================================================================
;
;disabchb: internal function

;
disabchb lda #nxpcrrsd|nxpcrrxd ;disable RxD & deassert RTS
;
;================================================================================
;
;chbctl: internal function
;
chbctl   sta io_acia+nx_crb    ;write to command register
         .rept 2               ;DUART reaction delay
           wai
         .endr
         rts
;
	.end