;================================================================================
;
;iabt: ABORT INTERRUPT SERVICE ROUTINE
;
iabt     phb                   ;save DB
         phd                   ;save DP
         longr                 ;16 bit registers
         pha
         phx
         phy
;
;
;ABORT REGISTER STACK FRAME
;
abt_yrx  .= 1                  ;.Y
abt_xrx  .= abt_yrx+s_word     ;.X
abt_arx  .= abt_xrx+s_word     ;.C
abt_dpx  .= abt_arx+s_word     ;DP
abt_dbx  .= abt_dpx+s_mpudpx   ;DB
abt_srx  .= abt_dbx+s_mpudbx   ;SR
abt_pcx  .= abt_srx+s_mpusrx   ;PC
abt_pbx  .= abt_pcx+s_mpupcx   ;PB
;
;
;         jmp (ivabt)           ;abort indirect vector
;
;	
;	If your system has an indirect vector for abort processing, it should
;	point to the IIRQA label immediately below.  Also, be sure to uncomment
;	the above instruction.  Use of the indirect vector allows you to patch
;	code into your ISR for development & testing purposes.  Please note
;	that the IVABT vector must be in RAM bank $00.
;	
;
;================================================================================
;
;iirq: HARDWARE INTERRUPT REQUEST SERVICE ROUTINE
;
;	
;	This interrupt service routine (ISR) processes UART interrupts & assum-
;	es that the 65C816 is in native mode.  Code unrelated to UART IRQs may
;	be added at location IIRB below.  More commentary on this follows.
;
;	Your 65C816 IRQ vector at $00FFEE-$00FFEF should be set to the address
;	of this ISR, which is defined by the label IIRQ (below).
;
;	This ISR starts by fully saving the MPU's state to assure foreground
;	transparency.  The UART part of the ISR uses all registers, so the ISR
;	preamble probably should not be changed in any way.
;
;	Due to the use of direct page for indexing & pointing, the UART ISR is
;	not fully reentrant.  Hence it is not recommended to enable IRQs within
;	the ISR.
;	
;
iirq     phb                   ;save DB
         phd                   ;save DP
         longr                 ;16 bit registers
         pha
         phx
         phy
;
;
;IRQ REGISTER STACK FRAME
;
irq_yrx  .= 1                  ;.Y
irq_xrx  .= irq_yrx+s_word     ;.X
irq_arx  .= irq_xrx+s_word     ;.C
irq_dpx  .= irq_arx+s_word     ;DP
irq_dbx  .= irq_dpx+s_mpudpx   ;DB
irq_srx  .= irq_dbx+s_mpudbx   ;SR
irq_pcx  .= irq_srx+s_mpusrx   ;PC
irq_pbx  .= irq_pcx+s_mpupcx   ;PB
;
;
;         jmp (ivirq)           ;IRQ indirect vector...
;
;	
;	If your system has an indirect vector for IRQ processing (recommended),
;	it should point to the IIRQA label immediately below.  Also, be sure to
;	uncomment the above instruction.  Use of the indirect vector allows you
;	to patch code into your ISR for development & testing purposes.  Please
;	note that the IVIRQ vector must be in RAM bank $00.
;	
;
iirqa    longa                 ;16 bit accumulator
         ldaw kerneldp         ;set default...
         tcd                   ;kernel DP
         shortr                ;8 bit registers
         lda #kerneldb         ;set...
         pha                   ;default...
         plb                   ;kernel DB
;
;---------------------------------------
;
;UART C/T IRQ PROCESSING
;
         lda io_acia+nx_isra   ;get block A IRQ status...
;
;	
;	Currently, only DUARTs are functional with this code  Eventually I will
;	write code that will work with the 28C94 & 28L192.
;	
;
         bne .iirq010          ;UART is interrupting
;
         jmp iirqb             ;UART is not interrupting...
;
;	
;	IIRQB is the continuation of the ISR, in which other interrupts unrelat-
;	ed to the UART would be serviced.  Obviously, this layout gives the UART 
;	highest IRQ priority, which in most cases is desirable.  If no other in-
;	terrupt sources are present no changes are needed at IIRQB.
;	
;
.iirq010 bit #nxpctirq         ;C/T interrupting?
         beq iirq01            ;no, must be I/O
;
;---------------------------------------
;
;UART COUNTER/TIMER IRQ PROCESSING
;
;	
;	This code segment assumes that the C/T in block A has been configured to
;	generate underflow IRQs.  As the DUARTs have only block A this doesn't
;	pose a problem at this time.
;	
;
         lda io_acia+nx_rcta   ;clear C/T interrupt
         longa
;
;	
;	This code segment updates the system up-time counter & the UNIX time
;	counter maintained in direct page storage.
;	
;
         ldx jiffct            ;clock jiffy counter
         inx                   ;increment
         cpx #hz
         bcc .iirq030          ;not time to update
;
         ldx #0                ;reset jiffy count
;
;
;	update system up-time counter...
;
         inc uptime            ;bump uptime LSW
         bne .iirq020          ;done with uptime
;
         inc uptime+s_word     ;bump uptime MSW
;
;
;	update UNIX time...
;
.iirq020 inc uxtime            ;bump UNIX time LSW
         bne .iirq030          ;done with UNIX time
;
         inc uxtime+s_word     ;bump UNIX time MID
         bne .iirq030          ;done with UNIX time
;
         inc uxtime+s_dword    ;bump UNIX time MSW
;
;
;	done with software timekeeping...
;
.iirq030 stx jiffct            ;set new jiffy count
;
;---------------------------------------
;
;UART RECEIVER IRQ PROCESSING
;
;	
;	This code segment services RxD IRQs by working backwards through the
;	configured UART blocks & channels.  Each channel has to be individually
;	checked to see if an RxD IRQ is pending.  This code is compatible with
;	the 26C92, 28L92 & 28C94.  It may be compatible with the 28L198 octal
;	UART, but has not been tested on that device.
;	
;
iirq01   shortr
         ldy #n_nxpblk-1       ;starting block index
;
;
;	block processing loop...
;
.iirq100 ldx #n_nxpch-1        ;starting channel index
;
;
;	channel processing loop...
;
.iirq110 phy                   ;save active block
         lda nxpirtab,y        ;get block's ISR...
         tay                   ;register number
         lda io_acia,y         ;read block IRQ status
         bit nxprqtab,x        ;RxD interrupting?
         beq .iirq140          ;no, skip this channel
;
         txa                   ;make a...
         asl                   ;table offset...
         tay                   ;from channel index
         longa
         lda nxpsrtab,y        ;set...
         sta tiaasr            ;active status register
         lda nxpfitab,y        ;set...
         sta tiaafif           ;active FIFO
         lda nxprbtab,y        ;set...
         sta tiaabuf           ;active buffer
         shorta
         ldy tiaputrx,x        ;get current buffer "put" index
;
;
;	FIFO processing loop...
;
.iirq120 lda (tiaasr)          ;get channel status
         bit #nxprxdr          ;RxD FIFO empty?
         beq .iirq130          ;yes, done with channel
;
         lda (tiaafif)         ;no, get datum &...
         xba                   ;hold it
         tya                   ;copy buffer "put" index,...
         ina                   ;bump it & wrap to...
         and #tiaxmask         ;buffer boundary
         cmp tiagetrx,x        ;check buffer status
         beq .iirq120          ;buffer full...
;
;	
;	If the buffer is full the datum will be lost, as it cannot be put back
;	into the FIFO.  The other approach to RxD processing would be to not
;	remove the datum from the FIFO until buffer space becomes available.
;	However, doing so will eventually cause the system to deadlock, as the
;	MPU will be continuously interrupted & thus won't be able to get any
;	foreground processing done.  This is because the only way the RxD IRQ
;	can be "silenced" is by reading the FIFO.
;	
;
         xba                   ;expose &...
         sta (tiaabuf),y       ;buffer datum
         xba
         tay                   ;new buffer "put" index
         bra .iirq120          ;get more data if available
;
.iirq130 sty tiaputrx,x        ;save new buffer "put" index
;
;
;	process next channel...
;
.iirq140 ply                   ;reload block index
         dex                   ;all channels serviced?
         bpl .iirq110          ;no, next
;
;
;	process next block...
;
         dey                   ;all blocks serviced?
         bpl .iirq100          ;no, do next block
;
;---------------------------------------
;
;UART TRANSMITTER IRQ PROCESSING
;
;	
;	This code segment services TxD IRQs by working backwards through the
;	configured UART blocks & channels.  Each channel has to be individually
;	checked to see if a TxD IRQ is pending.  This code is compatible with
;	the 26C92, 28L92 & 28C94.  It may be compatible with the 28L198 octal
;	UART, but has not been tested on that device.
;	
;
         ldy #n_nxpblk-1       ;starting block index
;
;
;	block processing loop...
;
.iirq200 ldx #n_nxpch-1        ;starting channel index
;
;
;	channel processing loop...
;
.iirq210 phy                   ;save active block
         lda nxpirtab,y        ;get block's ISR...
         tay                   ;register number
         lda io_acia,y         ;read block IRQ status
         bit nxptqtab,x        ;TxD interrupting?
         beq .iirq250          ;no, skip this channel
;
         txa                   ;make a...
         asl                   ;table offset...
         tay                   ;from channel index
         longa
         lda nxpcrtab,y        ;set
         sta tiaacr            ;active command register
         lda nxpsrtab,y        ;set...
         sta tiaasr            ;active status register
         lda nxpfitab,y        ;set...
         sta tiaafif           ;active FIFO
         lda nxptbtab,y        ;set...
         sta tiaabuf           ;active buffer
         shorta
         lda tiagettx,x        ;get current buffer "get" index &...
         bra .iirq230          ;enter FIFO processing loop
;
;
;	FIFO processing loop...
;
.iirq220 tay                   ;copy "get" index
         lda (tiaasr)          ;get channel status
         bit #nxptxdr          ;TxD FIFO full?
         beq .iirq240          ;yes, defer transmission...
;
;	
;	If the TxD FIFO is full it is likely the remote device has deasserted
;	its CTS output.  The UART will accept data into the TxD FIFO until it
;	is full, at which time TxD IRQs will cease.  Later, after the remote
;	device has asserted CTS, the transmitter will resume operation but will
;	not interrupt until the FIFO is empty.  So the process becomes one of
;	completely filling the FIFO, going on to something else until the FIFO
;	empties & then returning to repeat the process until the TxD buffer it-
;	self is empty.  The result of this algorithm is that only one IRQ will
;	occur per 8 (26C92) or 16 (28L92) bytes transmitted.
;	
;
         lda (tiaabuf),y       ;get datum from buffer &...
         sta (tiaafif)         ;write to FIFO
         iny                   ;new buffer "get" index
         tya
         and #tiaxmask         ;wrap to buffer boundary
;
.iirq230 cmp tiaputtx,x        ;data waiting in buffer?
         bne .iirq220          ;yes, process
;
         tay                   ;no, protect "get" index
         lda #nxpcrtxd         ;tell UART to...
         sta (tiaacr)          ;disable transmitter
         lda tiatstab,x        ;also tell foreground...
         tsb tiatxst           ;about it...
;
;	
;	If no data is buffered then the transmitter must be disabled so it won't
;	incessantly interrupt the MPU.  The foreground part of the driver will
;	bring the transmitter back on-line as soon as a datum has been stored in
;	the buffer.  Enabling the transmitter will cause it to immediately gen-
;	erate a TxD IRQ.  The bitfield at TIATXST indicates which of the trans-
;	mitters is disabled.
;	
;
;
;	process next channel...
;
.iirq240 sty tiagettx,x        ;save new buffer "get" index
;
.iirq250 ply                   ;get current block index
         dex                   ;all channels serviced?
         bpl .iirq210          ;no, do next channel
;
;
;	process next block...
;
         dey                   ;all blocks serviced?
         bpl .iirq200          ;no, do next block
;
;---------------------------------------
;
;INTERRUPT SERVICE ROUTINE CONTINUATION
;
;	
;	Add code as necessary at this point to service other interrupt sources
;	in your system.  Be sure to pay attention to register widths!
;	
;
iirqb   
;
;================================================================================
;
;crti: COMMON INTERRUPT RETURN
;
crti     longr                 ;16 bit registers
         ply                   ;recover...
         plx                   ;user...
         pla                   ;registers
         pld                   ;recover DP
         plb                   ;recover DB
         rti                   ;done with interrupts
;
;================================================================================
;
;inmi: NON-MASKABLE INTERRUPT SERVICE ROUTINE
;
inmi     phb                   ;save DB
         phd                   ;save DP
         longr                 ;16 bit registers
         pha
         phx
         phy
;
;
;NMI REGISTER STACK FRAME
;
nmi_yrx  .= 1                  ;.Y
nmi_xrx  .= nmi_yrx+s_word     ;.X
nmi_arx  .= nmi_xrx+s_word     ;.C
nmi_dpx  .= nmi_arx+s_word     ;DP
nmi_dbx  .= nmi_dpx+s_mpudpx   ;DB
nmi_srx  .= nmi_dbx+s_mpudbx   ;SR
nmi_pcx  .= nmi_srx+s_mpusrx   ;PC
nmi_pbx  .= nmi_pcx+s_mpupcx   ;PB
;
;
;         jmp (ivnmi)           ;NMI indirect vector
;
;	
;	Uncomment above if an indirect vector has been defined.  IVNMI should be
;	pointed to INMIA below.
;	
;
inmia    shorta
;
;	
;	The following code will cause control to be given to the monitor via the
;	BRK entry point if an NMI is detected while the MPU is executing code
;	outside of the ROM.  This should allow the user to regain control in the
;	event the MPU gets stuck in a loop.  If an NMI is detected while the MPU
;	is executing instructions in the ROM a silent return will occur.
;	
;
         ldas nmi_pbx          ;get interrupted bank
         longa
         bne .inmi010          ;apparently not in ROM
;
         ldas nmi_pcx          ;interrupted address
         cmpw mm_rom           ;anywhere in ROM?
         bcs crti              ;yes, ignore NMI, otherwise...
;
.inmi010 ;	jmp monbrk            ;goto M/L monitor
;
	.end