;================================================================================
;
;getcha: GET DATUM FROM TIA-232 CHANNEL A
;
;	
;	Preparatory Ops: NONE
;
;	Returned Values: .A: datum or entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: datum available
;	           |||||||      1: no datum
;	           |||++++> entry values
;	           ||+> 1
;	           ++> entry values
;
;	Example: JSR GETCHA
;	         BCS NODATUM
;	         STA DATUM
;	
;
getcha   php                   ;save register sizes
         longr                 ;16 bit registers
         phx                   ;save
         ldxw nxpchana         ;channel index
;
;================================================================================
;
;getchcom: internal function
;
;	
;	DO NOT directly call this function, as the stack will become unbalanced.
;	Only use the GETCHx functions, where x is the channel ID (A, B, etc.).
;	
;
getchcom phy                   ;preserve...
         pha                   ;remaining registers
         phb
         pha                   ;create workspace
;
;
.buf     =1                    ;buffer base address
.dbreg   =.buf+s_ptr           ;DB
.areg    =.dbreg+s_mpudbx      ;.C
.yreg    =.areg+s_word         ;.Y
.xreg    =.yreg+s_word         ;.X
.sreg    =.xreg+s_word         ;SR
;
;
         shortr
         lda #kerneldb
         pha
         plb                   ;set data bank
         lda tiagetrx,x        ;buffer "get" index
         cmp tiaputrx,x        ;buffer "put" index
         beq .0000010          ;same, buffer empty
;
         txa                   ;make a...
         asl                   ;table offset...
         tay                   ;from channel index
         longa
         lda nxprbtab,y        ;get & set...
         stas .buf             ;buffer base address
         shorta
         ldy tiagetrx,x        ;RxD buffer "get" index
         ldasi .buf            ;get from buffer &...
         stas .areg            ;overwrite .C's stack copy
         iny                   ;adjust buffer...
         tya                   ;"get" index
         and #tiaxmask         ;wrap to buffer boundary
         sta tiagetrx,x        ;new buffer "get" index
         clc                   ;datum avaliable
;
.0000010 ldas .sreg            ;SR stack copy
         and #sr_car_i         ;clear carry
         bcc .0000020          ;datum available
;
         ora #sr_car           ;no datum available
;
.0000020 ora #sr_amw           ;force 8 bit .A &...
         stas .sreg            ;condition SR
         longr
         pla                   ;discard workspace
         plb                   ;restore...
         pla                   ;registers
         ply
         plx
         plp
         rts
;
;================================================================================
;
;getchb: GET DATUM FROM TIA-232 CHANNEL B
;
;	
;	Preparatory Ops: NONE
;
;	Returned Values: .A: datum or entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: datum available
;	           |||||||      1: no datum
;	           |||++++> entry values
;	           ||+> 1
;	           ++> entry values
;
;	Notes: 1) See code comments in GETCHA for
;	          execution details.
;
;	Example: JSR GETCHB
;	         BCS NODATUM
;	         STA DATUM
;	
;
getchb   php                   ;save register sizes
         longr
         phx                   ;save
         ldxw nxpchanb         ;channel B index
         bra getchcom
;
	.if .ref(nx28c94)
;================================================================================
;
;getchc: GET DATUM FROM TIA-232 CHANNEL C
;
;	
;	Preparatory Ops: NONE
;
;	Returned Values: .A: datum or entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: datum available
;	           |||||||      1: no datum
;	           |||++++> entry values
;	           ||+> 1
;	           ++> entry values
;
;	Notes: 1) See code comments in GETCHA for
;	          execution details.
;
;	Example: JSR GETCHC
;	         BCS NODATUM
;	         STA DATUM
;	
;
getchc   php                   ;save register sizes
         longr
         phx                   ;save
         ldxw nxpchanc         ;channel C index
         bra getchcom
;
;================================================================================
;
;getchd: GET DATUM FROM TIA-232 CHANNEL D
;
;	
;	Preparatory Ops: NONE
;
;	Returned Values: .A: datum or entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0: datum available
;	           |||||||      1: no datum
;	           |||++++> entry values
;	           ||+> 1
;	           ++> entry values
;
;	Notes: 1) See code comments in GETCHA for
;	          execution details.
;
;	Example: JSR GETCHD
;	         BCS NODATUM
;	         STA DATUM
;	
;
getchd   php                   ;save register sizes
         longr
         phx                   ;save
         ldxw nxpchand         ;channel D index
         bra getchcom
;
	.endif
;================================================================================
;
;putcha: WRITE DATUM TO TIA-232 CHANNEL A
;
;	
;	Preparatory Ops: .A: 8 bit datum
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0
;	           ||||||+> entry value
;	           |||||+> 0
;	           +++++> entry values
;
;	Example: LDA #$41
;	         JSR PUTCHA
;
;	Notes: 1) This function will block if the buffer is full.
;	
;
putcha   php                   ;save MPU state
         longr                 ;16 bit registers
         phx                   ;save
         ldxw nxpchana         ;channel A index
;
;================================================================================
;
;putchcom: internal function
;
;	
;	DO NOT directly call this function, as the stack will become unbalanced.
;	Only use the PUTCHx functions, where x is the comm port ID.
;	
;
putchcom phy                   ;preserve...
         pha                   ;remaining registers
         phb
;
;
.buf     =1                    ;buffer base address
.chan    =.buf+s_ptr           ;channel base address
.dbreg   =.chan+s_ptr          ;DB
.areg    =.dbreg+s_mpudbx      ;.C
.yreg    =.areg+s_word         ;.Y
.xreg    =.yreg+s_word         ;.X
.sreg    =.xreg+s_word         ;SR
.s_tiaws =.dbreg-.buf          ;workspace needed
;
;
         sec
         tsc
         sbcw .s_tiaws         ;create workspace
         tcs
         shortr
         lda #kerneldb
         pha
         plb                   ;set data bank
         txa                   ;make a...
         asl                   ;table offset...
         tay                   ;from channel index
         longa
         lda nxptbtab,y        ;get & set...
         stas .buf             ;buffer base address
         lda nxpchtab,y        ;get & set...
         stas .chan            ;channel base address
         shorta
         ldy tiaputtx,x        ;buffer "put" index
         tya                   ;make a copy
         ina                   ;increment for space test
         and #tiaxmask         ;wrap to buffer boundary
;
.0000010 cmp tiagettx,x        ;TxD buffer "get" index
         beq .0000030          ;buffer full
;
         xba                   ;protect new index
         ldas .areg            ;get datum &...
         stasi .buf            ;buffer it
         xba                   ;recover new index &...
         sta tiaputtx,x        ;set it
         lda tiatstab,x        ;transmitter status bit
         trb tiatxst           ;transmitter enabled?
         beq .0000020          ;yes
;
         lda #nxpcrtxe         ;no
         ldy #nx_cr
         stasi .chan           ;enable transmitter
;
.0000020 longr
         clc
         tsc
         adcw .s_tiaws         ;discard workspace
         tcs
         plb                   ;restore...
         pla                   ;registers
         ply
         plx
         plp
         clc                   ;never an error
         rts
;
;
;	block for buffer space...
;
.0000030 wai
         bra .0000010
;
;================================================================================
;
;putchb: WRITE DATUM TO TIA-232 CHANNEL B
;
;	
;	Preparatory Ops: .A: 8 bit datum
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0
;	           ||||||+> entry value
;	           |||||+> 0
;	           +++++> entry values
;
;	Example: LDA #$41
;	         JSR PUTCHB
;
;	Notes: 1) This function will block if the buffer is full.
;	
;
putchb   php                   ;save MPU state
         longr                 ;16 bit registers
         phx                   ;save
         ldxw nxpchanb         ;channel B index
         bra putchcom
;
	.if .ref(nx28c94)
;================================================================================
;
;putchc: WRITE DATUM TO TIA-232 CHANNEL C
;
;	
;	Preparatory Ops: .A: 8 bit datum
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0
;	           ||||||+> entry value
;	           |||||+> 0
;	           +++++> entry values
;
;	Example: LDA #$41
;	         JSR PUTCHC
;
;	Notes: 1) This function will block if the buffer is full.
;	
;
putchc   php                   ;save MPU state
         longr                 ;16 bit registers
         phx                   ;save
         ldxw nxpchanc         ;channel C index
         bra putchcom
;
;================================================================================
;
;putchd: WRITE DATUM TO TIA-232 CHANNEL D
;
;	
;	Preparatory Ops: .A: 8 bit datum
;
;	Returned Values: .A: entry value
;	                 .B: entry value
;	                 .X: entry value
;	                 .Y: entry value
;
;	MPU Flags: NVmxDIZC
;	           ||||||||
;	           |||||||+> 0
;	           ||||||+> entry value
;	           |||||+> 0
;	           +++++> entry values
;
;	Example: LDA #$41
;	         JSR PUTCHD
;
;	Notes: 1) This function will block if the buffer is full.
;	
;
putchd   php                   ;save MPU state
         longr                 ;16 bit registers
         phx                   ;save
         ldxw nxpchanc         ;channel D index
         bra putchcom
;
	.endif
	.end