;================================================================================
;
;hrst: HARD SYSTEM RESET
;
hrst     sei                   ;no IRQs during 1st stage POST
         cld                   ;ensure binary mode
         sec                   ;select emulation mode to default...
         xce                   ;DB, DP & PB & register widths
;
;	
;	The above is redundant if this code is being executed because the MPU's
;	RESB input was negated.  However, if something does a JMP HRST the MPU's
;	state won't be known, so nothing can be assumed.
;	
;
         clc                   ;return to &...
         xce                   ;stay in native mode
;
;	
;	It may be necessary to set DB so the MPU sees the I/O hardware at the
;	MM_IO address instead of RAM.  I don't show that step here, but you can
;	add the code if need be.  Mapping I/O into something other than bank
;	zero isn't a particularly good idea...
;
;	It is recommended that a destructive test be performed on direct page &
;	hardware stack memory prior to doing any I/O device setup.  After I/O
;	has been set up then a full non-destructive memory test should be per-
;	formed to establish the amount of available RAM.
;	
;
         longa
         ldaw hwstack          ;top of hardware stack
         tcs                   ;set it
;
;	
;	Following a full non-destructive RAM test, you would install your indir-
;	ect system vectors.  They have to be in place before enabling interrupt
;	processing, as without them, the machine will crash.         
;	
;
;	do_vector_setup
;
;	
;	The following code performs setup on the UART & should be executed as
;	part of your system boot functions.  The UART setup procedure involves
;	reverse-reading a data table, in which each word consists of a chip reg-
;	ister number in the LSB & the parameter that is to be written into that
;	register in the MSB.  An approximate delay of 30 2 cycles is executed
;	after each register is written to give the UART sufficient time to proc-
;	ess.  This is more than enough delay to satisfy NXP's recommendations,
;	even at a 20 MHz 2 clock rate.
;	
;
         shortx                ;8 bit index
         ldy #s_nxptab-s_word  ;UART setup table index
;         
.0000010 longa                 ;16 bit accumulator
         lda nxpsutab,y        ;get register & parameter
         shorta                ;8 bit accumulator
         tax                   ;copy register
         xba                   ;expose parameter &...
         sta io_acia,x         ;write to register
         ldx #6                ;~30 clock cycles of...
;
.0000020 dex                   ;setup time delay (2 cycles)
         bne .0000020          ;                 (3 cycles)
;
         .rept s_word
             dey               ;decrement the table index
         .endr
         bpl .0000010          ;next
;
         lda io_acia+nx_sct    ;start the C/T &...
         cli                   ;allow IRQs...
;
;	
;	At this point, IRQ processing is live & C/T interrupts, which will occur
;	at 10 millisecond intervals, will start updating the system up-time &
;	UNIX time.  Also, TIA-232 processing will be live, which means that you
;	can start driving your console display at this point.
;	
;
;	...your reset code continues...
;
	.end